/**
 * @license
 * Copyright 2019 Google LLC
 * SPDX-License-Identifier: BSD-3-Clause
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { LitElement, html } from 'lit';
import { customElement, property, queryAssignedElements, } from 'lit/decorators.js';
/**
 * @summary highlights elements in a table of contents
 * if their counterparts are visible in the viewport.
 *
 * @slot toc - place for your table of contents, ul/ol with #-links
 *
 * Copyright © 2023 Tony Spegel
 */
let TocObserver = class TocObserver extends LitElement {
    constructor() {
        super(...arguments);
        // CSS class which is set when observer items are visible
        this.tocActiveClass = 'toc-active';
        // Bounding box which determines when an observation is triggered
        this.rootMargin = '0px';
        /**
         * Useful for observing nested markup like this:
         * <section>
         *   <!-- ^observe -->
         *   <h2 id="possum">Possum</h2>
         * </section>
         *
         * Observing wrapper elements instead of just headings
         * has the advantage that those have more area to intersect with.
         *
         *     ┌─────────┐
         *     │ #possum │
         *   ┌─┼─────────┼─┐
         *   │ │         │ │
         *   │ └─────────┘ │< Viewport
         *   │ ^section    │
         *   └─────────────┘
         */
        this.observeParent = false;
        // Should be used together with observeParent
        this.parentSelector = 'section';
    }
    /**
     * Selects a TOC link within the components slot / ul element.
     * Its id param is obtained by reading the oberver item's target-id.
     */
    selectTocLink(id) {
        var _a;
        return ((_a = this._tocList) === null || _a === void 0 ? void 0 : _a.length)
            ? this._tocList[0].querySelector(`[href="${id}"]`)
            : null;
    }
    /**
     * Receive any items within '_tocList' if present or null
     */
    get _tocListItems() {
        var _a;
        return ((_a = this._tocList) === null || _a === void 0 ? void 0 : _a.length)
            ? [...this._tocList[0].querySelectorAll('[href^="#"]')]
            : null;
    }
    /**
     * Creates a map of anchor-IDs and their corresponding
     * IntersectionObservers. Anchor-IDs are used to select
     * toc-links within '_tocList' and also to create observer
     * items later.
     *
     * {"#beschreibung" => IntersectionObserver}
     * {"#lebensweise" => IntersectionObserver}
     */
    createIdObserverMap(anchors) {
        return new Map(anchors.map((anchor) => {
            const { hash } = anchor;
            return [
                hash,
                new IntersectionObserver((entries) => {
                    entries.forEach((entry) => {
                        var _a, _b;
                        if (entry.intersectionRatio > 0) {
                            (_a = this.selectTocLink(hash)) === null || _a === void 0 ? void 0 : _a.classList.add(this.tocActiveClass);
                        }
                        else {
                            (_b = this.selectTocLink(hash)) === null || _b === void 0 ? void 0 : _b.classList.remove(this.tocActiveClass);
                        }
                    });
                }, 
                // IntersectionObserver options
                {
                    root: this.ownerDocument.querySelector(this.rootElement) || null,
                    rootMargin: this.rootMargin,
                }),
            ];
        }));
    }
    /**
     * The 'firstUpdated' lifecycle is called after the component's DOM
     * has been updated the first time, immediately before updated() is called.
     * Only then an element's slot content (our toc items) is available and can be observed.
     */
    firstUpdated() {
        var _a;
        // Observe items when at least one is available
        if ((_a = this._tocListItems) === null || _a === void 0 ? void 0 : _a.length) {
            this.anchorHashObserverMap = this.createIdObserverMap(this._tocListItems);
            this.anchorHashObserverMap.forEach((observer, anchorHash) => {
                const item = this.ownerDocument.querySelector(anchorHash);
                if (this.observeParent === false && item) {
                    observer.observe(item);
                }
                if (this.observeParent === true &&
                    item.closest(this.parentSelector) !== null) {
                    observer.observe(item.closest(this.parentSelector));
                }
            });
        }
    }
    /**
     * Stop observing when the component is removed from the DOM
     */
    disconnectedCallback() {
        // As there are no toc-items left to highlight observing
        // elements should be stopped
        this.anchorHashObserverMap.forEach((obs) => obs.disconnect());
    }
    render() {
        return html `<slot name="toc"></slot>`;
    }
};
__decorate([
    property({ type: String })
], TocObserver.prototype, "tocActiveClass", void 0);
__decorate([
    property({ type: String })
], TocObserver.prototype, "rootElement", void 0);
__decorate([
    property({ type: String })
], TocObserver.prototype, "rootMargin", void 0);
__decorate([
    property({ type: Boolean })
], TocObserver.prototype, "observeParent", void 0);
__decorate([
    property({ type: String })
], TocObserver.prototype, "parentSelector", void 0);
__decorate([
    queryAssignedElements({ slot: 'toc' })
], TocObserver.prototype, "_tocList", void 0);
TocObserver = __decorate([
    customElement('toc-observer')
], TocObserver);
export { TocObserver };
//# sourceMappingURL=toc-observer.js.map